/**
 * vis-graph3d
 * https://visjs.github.io/vis-graph3d/
 *
 * Create interactive, animated 3d graphs. Surfaces, lines, dots and block styling out of the box.
 *
 * @version 6.0.3
 * @date    2023-03-21T17:15:34.624Z
 *
 * @copyright (c) 2011-2017 Almende B.V, http://almende.com
 * @copyright (c) 2017-2019 visjs contributors, https://github.com/visjs
 *
 * @license
 * vis.js is dual licensed under both
 *
 *   1. The Apache 2.0 License
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *   and
 *
 *   2. The MIT License
 *      http://opensource.org/licenses/MIT
 *
 * vis.js may be distributed under either license.
 */
import t from"component-emitter";import*as e from"vis-util/esnext/esm/vis-util.js";import{Validator as i,VALIDATOR_PRINT_STYLE as o}from"vis-util/esnext/esm/vis-util.js";import{DataView as n,DataSet as s}from"vis-data/esnext/esm/vis-data.js";function a(t,e,i){this.x=void 0!==t?t:0,this.y=void 0!==e?e:0,this.z=void 0!==i?i:0}a.subtract=function(t,e){const i=new a;return i.x=t.x-e.x,i.y=t.y-e.y,i.z=t.z-e.z,i},a.add=function(t,e){const i=new a;return i.x=t.x+e.x,i.y=t.y+e.y,i.z=t.z+e.z,i},a.avg=function(t,e){return new a((t.x+e.x)/2,(t.y+e.y)/2,(t.z+e.z)/2)},a.scalarProduct=function(t,e){return new a(t.x*e,t.y*e,t.z*e)},a.dotProduct=function(t,e){return t.x*e.x+t.y*e.y+t.z*e.z},a.crossProduct=function(t,e){const i=new a;return i.x=t.y*e.z-t.z*e.y,i.y=t.z*e.x-t.x*e.z,i.z=t.x*e.y-t.y*e.x,i},a.prototype.length=function(){return Math.sqrt(this.x*this.x+this.y*this.y+this.z*this.z)},a.prototype.normalize=function(){return a.scalarProduct(this,1/this.length())};var r=a;var h=function(t,e){this.x=void 0!==t?t:0,this.y=void 0!==e?e:0};function l(t,e){if(void 0===t)throw new Error("No container element defined");if(this.container=t,this.visible=!e||null==e.visible||e.visible,this.visible){this.frame=document.createElement("DIV"),this.frame.style.width="100%",this.frame.style.position="relative",this.container.appendChild(this.frame),this.frame.prev=document.createElement("INPUT"),this.frame.prev.type="BUTTON",this.frame.prev.value="Prev",this.frame.appendChild(this.frame.prev),this.frame.play=document.createElement("INPUT"),this.frame.play.type="BUTTON",this.frame.play.value="Play",this.frame.appendChild(this.frame.play),this.frame.next=document.createElement("INPUT"),this.frame.next.type="BUTTON",this.frame.next.value="Next",this.frame.appendChild(this.frame.next),this.frame.bar=document.createElement("INPUT"),this.frame.bar.type="BUTTON",this.frame.bar.style.position="absolute",this.frame.bar.style.border="1px solid red",this.frame.bar.style.width="100px",this.frame.bar.style.height="6px",this.frame.bar.style.borderRadius="2px",this.frame.bar.style.MozBorderRadius="2px",this.frame.bar.style.border="1px solid #7F7F7F",this.frame.bar.style.backgroundColor="#E5E5E5",this.frame.appendChild(this.frame.bar),this.frame.slide=document.createElement("INPUT"),this.frame.slide.type="BUTTON",this.frame.slide.style.margin="0px",this.frame.slide.value=" ",this.frame.slide.style.position="relative",this.frame.slide.style.left="-100px",this.frame.appendChild(this.frame.slide);const t=this;this.frame.slide.onmousedown=function(e){t._onMouseDown(e)},this.frame.prev.onclick=function(e){t.prev(e)},this.frame.play.onclick=function(e){t.togglePlay(e)},this.frame.next.onclick=function(e){t.next(e)}}this.onChangeCallback=void 0,this.values=[],this.index=void 0,this.playTimeout=void 0,this.playInterval=1e3,this.playLoop=!0}function c(t,e,i,o){this._start=0,this._end=0,this._step=1,this.prettyStep=!0,this.precision=5,this._current=0,this.setRange(t,e,i,o)}l.prototype.prev=function(){let t=this.getIndex();t>0&&(t--,this.setIndex(t))},l.prototype.next=function(){let t=this.getIndex();t<this.values.length-1&&(t++,this.setIndex(t))},l.prototype.playNext=function(){const t=new Date;let e=this.getIndex();e<this.values.length-1?(e++,this.setIndex(e)):this.playLoop&&(e=0,this.setIndex(e));const i=new Date-t,o=Math.max(this.playInterval-i,0),n=this;this.playTimeout=setTimeout((function(){n.playNext()}),o)},l.prototype.togglePlay=function(){void 0===this.playTimeout?this.play():this.stop()},l.prototype.play=function(){this.playTimeout||(this.playNext(),this.frame&&(this.frame.play.value="Stop"))},l.prototype.stop=function(){clearInterval(this.playTimeout),this.playTimeout=void 0,this.frame&&(this.frame.play.value="Play")},l.prototype.setOnChangeCallback=function(t){this.onChangeCallback=t},l.prototype.setPlayInterval=function(t){this.playInterval=t},l.prototype.getPlayInterval=function(){return this.playInterval},l.prototype.setPlayLoop=function(t){this.playLoop=t},l.prototype.onChange=function(){void 0!==this.onChangeCallback&&this.onChangeCallback()},l.prototype.redraw=function(){if(this.frame){this.frame.bar.style.top=this.frame.clientHeight/2-this.frame.bar.offsetHeight/2+"px",this.frame.bar.style.width=this.frame.clientWidth-this.frame.prev.clientWidth-this.frame.play.clientWidth-this.frame.next.clientWidth-30+"px";const t=this.indexToLeft(this.index);this.frame.slide.style.left=t+"px"}},l.prototype.setValues=function(t){this.values=t,this.values.length>0?this.setIndex(0):this.index=void 0},l.prototype.setIndex=function(t){if(!(t<this.values.length))throw new Error("Index out of range");this.index=t,this.redraw(),this.onChange()},l.prototype.getIndex=function(){return this.index},l.prototype.get=function(){return this.values[this.index]},l.prototype._onMouseDown=function(t){if(!(t.which?1===t.which:1===t.button))return;this.startClientX=t.clientX,this.startSlideX=parseFloat(this.frame.slide.style.left),this.frame.style.cursor="move";const i=this;this.onmousemove=function(t){i._onMouseMove(t)},this.onmouseup=function(t){i._onMouseUp(t)},e.addEventListener(document,"mousemove",this.onmousemove),e.addEventListener(document,"mouseup",this.onmouseup),e.preventDefault(t)},l.prototype.leftToIndex=function(t){const e=parseFloat(this.frame.bar.style.width)-this.frame.slide.clientWidth-10,i=t-3;let o=Math.round(i/e*(this.values.length-1));return o<0&&(o=0),o>this.values.length-1&&(o=this.values.length-1),o},l.prototype.indexToLeft=function(t){const e=parseFloat(this.frame.bar.style.width)-this.frame.slide.clientWidth-10;return t/(this.values.length-1)*e+3},l.prototype._onMouseMove=function(t){const i=t.clientX-this.startClientX,o=this.startSlideX+i,n=this.leftToIndex(o);this.setIndex(n),e.preventDefault()},l.prototype._onMouseUp=function(){this.frame.style.cursor="auto",e.removeEventListener(document,"mousemove",this.onmousemove),e.removeEventListener(document,"mouseup",this.onmouseup),e.preventDefault()},c.prototype.isNumeric=function(t){return!isNaN(parseFloat(t))&&isFinite(t)},c.prototype.setRange=function(t,e,i,o){if(!this.isNumeric(t))throw new Error("Parameter 'start' is not numeric; value: "+t);if(!this.isNumeric(e))throw new Error("Parameter 'end' is not numeric; value: "+t);if(!this.isNumeric(i))throw new Error("Parameter 'step' is not numeric; value: "+t);this._start=t||0,this._end=e||0,this.setStep(i,o)},c.prototype.setStep=function(t,e){void 0===t||t<=0||(void 0!==e&&(this.prettyStep=e),!0===this.prettyStep?this._step=c.calculatePrettyStep(t):this._step=t)},c.calculatePrettyStep=function(t){const e=function(t){return Math.log(t)/Math.LN10},i=Math.pow(10,Math.round(e(t))),o=2*Math.pow(10,Math.round(e(t/2))),n=5*Math.pow(10,Math.round(e(t/5)));let s=i;return Math.abs(o-t)<=Math.abs(s-t)&&(s=o),Math.abs(n-t)<=Math.abs(s-t)&&(s=n),s<=0&&(s=1),s},c.prototype.getCurrent=function(){return parseFloat(this._current.toPrecision(this.precision))},c.prototype.getStep=function(){return this._step},c.prototype.start=function(t){void 0===t&&(t=!1),this._current=this._start-this._start%this._step,t&&this.getCurrent()<this._start&&this.next()},c.prototype.next=function(){this._current+=this._step},c.prototype.end=function(){return this._current>this._end};var d=c;function u(){this.armLocation=new r,this.armRotation={},this.armRotation.horizontal=0,this.armRotation.vertical=0,this.armLength=1.7,this.cameraOffset=new r,this.offsetMultiplier=.6,this.cameraLocation=new r,this.cameraRotation=new r(.5*Math.PI,0,0),this.calculateCameraOrientation()}u.prototype.setOffset=function(t,e){const i=Math.abs,o=Math.sign,n=this.offsetMultiplier,s=this.armLength*n;i(t)>s&&(t=o(t)*s),i(e)>s&&(e=o(e)*s),this.cameraOffset.x=t,this.cameraOffset.y=e,this.calculateCameraOrientation()},u.prototype.getOffset=function(){return this.cameraOffset},u.prototype.setArmLocation=function(t,e,i){this.armLocation.x=t,this.armLocation.y=e,this.armLocation.z=i,this.calculateCameraOrientation()},u.prototype.setArmRotation=function(t,e){void 0!==t&&(this.armRotation.horizontal=t),void 0!==e&&(this.armRotation.vertical=e,this.armRotation.vertical<0&&(this.armRotation.vertical=0),this.armRotation.vertical>.5*Math.PI&&(this.armRotation.vertical=.5*Math.PI)),void 0===t&&void 0===e||this.calculateCameraOrientation()},u.prototype.getArmRotation=function(){const t={};return t.horizontal=this.armRotation.horizontal,t.vertical=this.armRotation.vertical,t},u.prototype.setArmLength=function(t){void 0!==t&&(this.armLength=t,this.armLength<.71&&(this.armLength=.71),this.armLength>5&&(this.armLength=5),this.setOffset(this.cameraOffset.x,this.cameraOffset.y),this.calculateCameraOrientation())},u.prototype.getArmLength=function(){return this.armLength},u.prototype.getCameraLocation=function(){return this.cameraLocation},u.prototype.getCameraRotation=function(){return this.cameraRotation},u.prototype.calculateCameraOrientation=function(){this.cameraLocation.x=this.armLocation.x-this.armLength*Math.sin(this.armRotation.horizontal)*Math.cos(this.armRotation.vertical),this.cameraLocation.y=this.armLocation.y-this.armLength*Math.cos(this.armRotation.horizontal)*Math.cos(this.armRotation.vertical),this.cameraLocation.z=this.armLocation.z+this.armLength*Math.sin(this.armRotation.vertical),this.cameraRotation.x=Math.PI/2-this.armRotation.vertical,this.cameraRotation.y=0,this.cameraRotation.z=-this.armRotation.horizontal;const t=this.cameraRotation.x,e=this.cameraRotation.z,i=this.cameraOffset.x,o=this.cameraOffset.y,n=Math.sin,s=Math.cos;this.cameraLocation.x=this.cameraLocation.x+i*s(e)+o*-n(e)*s(t),this.cameraLocation.y=this.cameraLocation.y+i*n(e)+o*s(e)*s(t),this.cameraLocation.z=this.cameraLocation.z+o*n(t)};const p={BAR:0,BARCOLOR:1,BARSIZE:2,DOT:3,DOTLINE:4,DOTCOLOR:5,DOTSIZE:6,GRID:7,LINE:8,SURFACE:9},f={dot:p.DOT,"dot-line":p.DOTLINE,"dot-color":p.DOTCOLOR,"dot-size":p.DOTSIZE,line:p.LINE,grid:p.GRID,surface:p.SURFACE,bar:p.BAR,"bar-color":p.BARCOLOR,"bar-size":p.BARSIZE},m=["width","height","filterLabel","legendLabel","xLabel","yLabel","zLabel","xValueLabel","yValueLabel","zValueLabel","showXAxis","showYAxis","showZAxis","showGrayBottom","showGrid","showPerspective","showShadow","showSurfaceGrid","keepAspectRatio","rotateAxisLabels","verticalRatio","dotSizeRatio","dotSizeMinFraction","dotSizeMaxFraction","showAnimationControls","animationInterval","animationPreload","animationAutoStart","axisColor","axisFontSize","axisFontType","gridColor","xCenter","yCenter","zoomable","tooltipDelay","ctrlToZoom"],y=["xBarWidth","yBarWidth","valueMin","valueMax","xMin","xMax","xStep","yMin","yMax","yStep","zMin","zMax","zStep"];let g;function x(t){for(const e in t)if(Object.prototype.hasOwnProperty.call(t,e))return!1;return!0}function v(t,e){return void 0===t||""===t?e:t+(void 0===(i=e)||""===i||"string"!=typeof i?i:i.charAt(0).toUpperCase()+i.slice(1));var i}function w(t,e,i,o){let n,s;for(let a=0;a<i.length;++a)n=i[a],s=v(o,n),e[s]=t[n]}function b(t,e,i,o){let n,s;for(let a=0;a<i.length;++a)n=i[a],void 0!==t[n]&&(s=v(o,n),e[s]=t[n])}function _(t,i){if(void 0!==t.backgroundColor&&function(t,e){let i="white",o="gray",n=1;if("string"==typeof t)i=t,o="none",n=0;else{if("object"!=typeof t)throw new Error("Unsupported type of backgroundColor");void 0!==t.fill&&(i=t.fill),void 0!==t.stroke&&(o=t.stroke),void 0!==t.strokeWidth&&(n=t.strokeWidth)}e.frame.style.backgroundColor=i,e.frame.style.borderColor=o,e.frame.style.borderWidth=n+"px",e.frame.style.borderStyle="solid"}(t.backgroundColor,i),function(t,e){if(void 0===t)return;void 0===e.dataColor&&(e.dataColor={});"string"==typeof t?(e.dataColor.fill=t,e.dataColor.stroke=t):(t.fill&&(e.dataColor.fill=t.fill),t.stroke&&(e.dataColor.stroke=t.stroke),void 0!==t.strokeWidth&&(e.dataColor.strokeWidth=t.strokeWidth))}(t.dataColor,i),function(t,e){if(void 0===t)return;let i;if("string"==typeof t){if(i=function(t){const e=f[t];if(void 0===e)return-1;return e}(t),-1===i)throw new Error("Style '"+t+"' is invalid")}else{if(!function(t){let e=!1;for(const i in p)if(p[i]===t){e=!0;break}return e}(t))throw new Error("Style '"+t+"' is invalid");i=t}e.style=i}(t.style,i),void 0!==t.surfaceColors){if(console.warn("`options.surfaceColors` is deprecated and may be removed in a future version. Please use `options.colormap` instead. Note that the `colormap` option uses the inverse array ordering (running from vMin to vMax)."),void 0!==t.colormap)throw new Error("The `colormap` and `surfaceColors` options are mutually exclusive.");"surface"!==i.style?console.warn("Ignoring `surfaceColors` in graph style `"+i.style+"` for backward compatibility (only effective in `surface` plots)."):function(t,e){if(void 0===t||!0===t)return;if(!1===t)return void(e.surfaceColors=void 0);void 0===e.surfaceColors&&(e.surfaceColors={});let i;if(Array.isArray(t))i=C(t);else{if("object"!=typeof t)throw new Error("Unsupported type of surfaceColors");i=L(t.hue)}i.reverse(),e.colormap=i}(t.surfaceColors,i)}else!function(t,e){if(void 0===t)return;let i;if(Array.isArray(t))i=C(t);else if("object"==typeof t)i=L(t.hue);else{if("function"!=typeof t)throw new Error("Unsupported type of colormap");i=t}e.colormap=i}(t.colormap,i);!function(t,e){if(void 0===t){if(void 0===g.showLegend){const t=e.style===p.DOTCOLOR||e.style===p.DOTSIZE;e.showLegend=t}}else e.showLegend=t}(t.showLegend,i),S(t.cameraPosition,i),void 0!==t.tooltip&&(i.showTooltip=t.tooltip),null!=t.onclick&&(i.onclick_callback=t.onclick,console.warn("`options.onclick` is deprecated and may be removed in a future version. Please use `Graph3d.on('click', handler)` instead.")),void 0!==t.tooltipStyle&&e.selectiveDeepExtend(["tooltipStyle"],i,t)}function C(t){if(t.length<2)throw new Error("Colormap array length must be 2 or above.");return t.map((function(t){if(!e.isValidHex(t))throw new Error("Invalid hex color code supplied to colormap.");return e.hexToRGB(t)}))}function L(t){if(void 0===t)throw new Error("Unsupported type of colormap");if(!(t.saturation>=0&&t.saturation<=100))throw new Error("Saturation is out of bounds. Expected range is 0-100.");if(!(t.brightness>=0&&t.brightness<=100))throw new Error("Brightness is out of bounds. Expected range is 0-100.");if(!(t.colorStops>=2))throw new Error("colorStops is out of bounds. Expected 2 or above.");const i=(t.end-t.start)/(t.colorStops-1),o=[];for(let n=0;n<t.colorStops;++n){const s=(t.start+i*n)%360/360;o.push(e.HSVToRGB(s<0?s+1:s,t.saturation/100,t.brightness/100))}return o}function S(t,e){const i=t;void 0!==i&&(void 0===e.camera&&(e.camera=new u),e.camera.setArmRotation(i.horizontal,i.vertical),e.camera.setArmLength(i.distance))}const T="string",R="boolean",M="number",z="object",D="array",E={fill:{string:T},stroke:{string:T},strokeWidth:{number:M},__type__:{string:T,object:z,undefined:"undefined"}},P={animationAutoStart:{boolean:R,undefined:"undefined"},animationInterval:{number:M},animationPreload:{boolean:R},axisColor:{string:T},axisFontSize:{number:M},axisFontType:{string:T},backgroundColor:E,xBarWidth:{number:M,undefined:"undefined"},yBarWidth:{number:M,undefined:"undefined"},cameraPosition:{distance:{number:M},horizontal:{number:M},vertical:{number:M},__type__:{object:z}},zoomable:{boolean:R},ctrlToZoom:{boolean:R},xCenter:{string:T},yCenter:{string:T},colormap:{hue:{start:{number:M},end:{number:M},saturation:{number:M},brightness:{number:M},colorStops:{number:M},__type__:{object:z}},__type__:{array:D,object:z,function:"function",undefined:"undefined"}},dataColor:E,dotSizeMinFraction:{number:M},dotSizeMaxFraction:{number:M},dotSizeRatio:{number:M},filterLabel:{string:T},gridColor:{string:T},onclick:{function:"function"},keepAspectRatio:{boolean:R},xLabel:{string:T},yLabel:{string:T},zLabel:{string:T},legendLabel:{string:T},xMin:{number:M,undefined:"undefined"},yMin:{number:M,undefined:"undefined"},zMin:{number:M,undefined:"undefined"},xMax:{number:M,undefined:"undefined"},yMax:{number:M,undefined:"undefined"},zMax:{number:M,undefined:"undefined"},showAnimationControls:{boolean:R,undefined:"undefined"},showGrayBottom:{boolean:R},showGrid:{boolean:R},showLegend:{boolean:R,undefined:"undefined"},showPerspective:{boolean:R},showShadow:{boolean:R},showSurfaceGrid:{boolean:R},showXAxis:{boolean:R},showYAxis:{boolean:R},showZAxis:{boolean:R},rotateAxisLabels:{boolean:R},surfaceColors:{hue:{start:{number:M},end:{number:M},saturation:{number:M},brightness:{number:M},colorStops:{number:M},__type__:{object:z}},__type__:{boolean:R,array:D,object:z,undefined:"undefined"}},xStep:{number:M,undefined:"undefined"},yStep:{number:M,undefined:"undefined"},zStep:{number:M,undefined:"undefined"},style:{number:M,string:["bar","bar-color","bar-size","dot","dot-line","dot-color","dot-size","line","grid","surface"]},tooltip:{boolean:R,function:"function"},tooltipDelay:{number:M},tooltipStyle:{content:{color:{string:T},background:{string:T},border:{string:T},borderRadius:{string:T},boxShadow:{string:T},padding:{string:T},__type__:{object:z}},line:{borderLeft:{string:T},height:{string:T},width:{string:T},pointerEvents:{string:T},__type__:{object:z}},dot:{border:{string:T},borderRadius:{string:T},height:{string:T},width:{string:T},pointerEvents:{string:T},__type__:{object:z}},__type__:{object:z}},xValueLabel:{function:"function"},yValueLabel:{function:"function"},zValueLabel:{function:"function"},valueMax:{number:M,undefined:"undefined"},valueMin:{number:M,undefined:"undefined"},verticalRatio:{number:M},height:{string:T},width:{string:T},__type__:{object:z}};function A(){this.min=void 0,this.max=void 0}A.prototype.adjust=function(t){void 0!==t&&((void 0===this.min||this.min>t)&&(this.min=t),(void 0===this.max||this.max<t)&&(this.max=t))},A.prototype.combine=function(t){this.add(t.min),this.add(t.max)},A.prototype.expand=function(t){if(void 0===t)return;const e=this.min-t,i=this.max+t;if(e>i)throw new Error("Passed expansion value makes range invalid");this.min=e,this.max=i},A.prototype.range=function(){return this.max-this.min},A.prototype.center=function(){return(this.min+this.max)/2};var k=A;function B(t,e,i){this.dataGroup=t,this.column=e,this.graph=i,this.index=void 0,this.value=void 0,this.values=t.getDistinctValues(this.column),this.values.length>0&&this.selectValue(0),this.dataPoints=[],this.loaded=!1,this.onLoadCallback=void 0,i.animationPreload?(this.loaded=!1,this.loadInBackground()):this.loaded=!0}function I(){this.dataTable=null}B.prototype.isLoaded=function(){return this.loaded},B.prototype.getLoadedProgress=function(){const t=this.values.length;let e=0;for(;this.dataPoints[e];)e++;return Math.round(e/t*100)},B.prototype.getLabel=function(){return this.graph.filterLabel},B.prototype.getColumn=function(){return this.column},B.prototype.getSelectedValue=function(){if(void 0!==this.index)return this.values[this.index]},B.prototype.getValues=function(){return this.values},B.prototype.getValue=function(t){if(t>=this.values.length)throw new Error("Index out of range");return this.values[t]},B.prototype._getDataPoints=function(t){if(void 0===t&&(t=this.index),void 0===t)return[];let e;if(this.dataPoints[t])e=this.dataPoints[t];else{const i={};i.column=this.column,i.value=this.values[t];const o=new n(this.dataGroup.getDataSet(),{filter:function(t){return t[i.column]==i.value}}).get();e=this.dataGroup._getDataPoints(o),this.dataPoints[t]=e}return e},B.prototype.setOnLoadCallback=function(t){this.onLoadCallback=t},B.prototype.selectValue=function(t){if(t>=this.values.length)throw new Error("Index out of range");this.index=t,this.value=this.values[t]},B.prototype.loadInBackground=function(t){void 0===t&&(t=0);const e=this.graph.frame;if(t<this.values.length){void 0===e.progress&&(e.progress=document.createElement("DIV"),e.progress.style.position="absolute",e.progress.style.color="gray",e.appendChild(e.progress));const i=this.getLoadedProgress();e.progress.innerHTML="Loading animation... "+i+"%",e.progress.style.bottom="60px",e.progress.style.left="10px";const o=this;setTimeout((function(){o.loadInBackground(t+1)}),10),this.loaded=!1}else this.loaded=!0,void 0!==e.progress&&(e.removeChild(e.progress),e.progress=void 0),this.onLoadCallback&&this.onLoadCallback()},I.prototype.initializeData=function(t,e,i){if(void 0===e)return;let o;if(Array.isArray(e)&&(e=new s(e)),!(e instanceof s||e instanceof n))throw new Error("Array, DataSet, or DataView expected");if(o=e.get(),0==o.length)return;this.style=i,this.dataSet&&this.dataSet.off("*",this._onChange),this.dataSet=e,this.dataTable=o;const a=this;this._onChange=function(){t.setData(a.dataSet)},this.dataSet.on("*",this._onChange),this.colX="x",this.colY="y",this.colZ="z";const r=t.hasBars(i);if(r&&(void 0!==t.defaultXBarWidth?this.xBarWidth=t.defaultXBarWidth:this.xBarWidth=this.getSmallestDifference(o,this.colX)||1,void 0!==t.defaultYBarWidth?this.yBarWidth=t.defaultYBarWidth:this.yBarWidth=this.getSmallestDifference(o,this.colY)||1),this._initializeRange(o,this.colX,t,r),this._initializeRange(o,this.colY,t,r),this._initializeRange(o,this.colZ,t,!1),Object.prototype.hasOwnProperty.call(o[0],"style")){this.colValue="style";const e=this.getColumnRange(o,this.colValue);this._setRangeDefaults(e,t.defaultValueMin,t.defaultValueMax),this.valueRange=e}else this.colValue="z",this.valueRange=this.zRange;const h=this.getDataTable();let l;return Object.prototype.hasOwnProperty.call(h[0],"filter")&&void 0===this.dataFilter&&(this.dataFilter=new B(this,"filter",t),this.dataFilter.setOnLoadCallback((function(){t.redraw()}))),l=this.dataFilter?this.dataFilter._getDataPoints():this._getDataPoints(this.getDataTable()),l},I.prototype._collectRangeSettings=function(t,e){if(-1==["x","y","z"].indexOf(t))throw new Error("Column '"+t+"' invalid");const i=t.toUpperCase();return{barWidth:this[t+"BarWidth"],min:e["default"+i+"Min"],max:e["default"+i+"Max"],step:e["default"+i+"Step"],range_label:t+"Range",step_label:t+"Step"}},I.prototype._initializeRange=function(t,e,i,o){const n=this._collectRangeSettings(e,i),s=this.getColumnRange(t,e);o&&"z"!=e&&s.expand(n.barWidth/2),this._setRangeDefaults(s,n.min,n.max),this[n.range_label]=s,this[n.step_label]=void 0!==n.step?n.step:s.range()/5},I.prototype.getDistinctValues=function(t,e){void 0===e&&(e=this.dataTable);const i=[];for(let o=0;o<e.length;o++){const n=e[o][t]||0;-1===i.indexOf(n)&&i.push(n)}return i.sort((function(t,e){return t-e}))},I.prototype.getSmallestDifference=function(t,e){const i=this.getDistinctValues(t,e);let o=null;for(let t=1;t<i.length;t++){const e=i[t]-i[t-1];(null==o||o>e)&&(o=e)}return o},I.prototype.getColumnRange=function(t,e){const i=new k;for(let o=0;o<t.length;o++){const n=t[o][e];i.adjust(n)}return i},I.prototype.getNumberOfRows=function(){return this.dataTable.length},I.prototype._setRangeDefaults=function(t,e,i){void 0!==e&&(t.min=e),void 0!==i&&(t.max=i),t.max<=t.min&&(t.max=t.min+1)},I.prototype.getDataTable=function(){return this.dataTable},I.prototype.getDataSet=function(){return this.dataSet},I.prototype.getDataPoints=function(t){const e=[];for(let i=0;i<t.length;i++){const o=new r;o.x=t[i][this.colX]||0,o.y=t[i][this.colY]||0,o.z=t[i][this.colZ]||0,o.data=t[i],o.value=t[i][this.colValue]||0;const n={};n.point=o,n.bottom=new r(o.x,o.y,this.zRange.min),n.trans=void 0,n.screen=void 0,e.push(n)}return e},I.prototype.initDataAsMatrix=function(t){let e,i,o,n;const s=this.getDistinctValues(this.colX,t),a=this.getDistinctValues(this.colY,t),r=this.getDataPoints(t),h=[];for(o=0;o<r.length;o++){n=r[o];const t=s.indexOf(n.point.x),e=a.indexOf(n.point.y);void 0===h[t]&&(h[t]=[]),h[t][e]=n}for(e=0;e<h.length;e++)for(i=0;i<h[e].length;i++)h[e][i]&&(h[e][i].pointRight=e<h.length-1?h[e+1][i]:void 0,h[e][i].pointTop=i<h[e].length-1?h[e][i+1]:void 0,h[e][i].pointCross=e<h.length-1&&i<h[e].length-1?h[e+1][i+1]:void 0);return r},I.prototype.getInfo=function(){const t=this.dataFilter;if(t)return t.getLabel()+": "+t.getSelectedValue()},I.prototype.reload=function(){this.dataTable&&this.setData(this.dataTable)},I.prototype._getDataPoints=function(t){let e=[];if(this.style===p.GRID||this.style===p.SURFACE)e=this.initDataAsMatrix(t);else if(e=this.getDataPoints(t),this.style===p.LINE)for(let t=0;t<e.length;t++)t>0&&(e[t-1].pointNext=e[t]);return e},W.STYLE=p;const O=void 0;function W(t,e,i){if(!(this instanceof W))throw new SyntaxError("Constructor must be called with the new operator");this.containerElement=t,this.dataGroup=new I,this.dataPoints=null,this.create(),function(t,e){if(void 0===t||x(t))throw new Error("No DEFAULTS passed");if(void 0===e)throw new Error("No dst passed");g=t,w(t,e,m),w(t,e,y,"default"),_(t,e),e.margin=10,e.showTooltip=!1,e.onclick_callback=null,e.eye=new r(0,0,-1)}(W.DEFAULTS,this),this.colX=void 0,this.colY=void 0,this.colZ=void 0,this.colValue=void 0,this.setOptions(i),this.setData(e)}function G(t){return"clientX"in t?t.clientX:t.targetTouches[0]&&t.targetTouches[0].clientX||0}function Y(t){return"clientY"in t?t.clientY:t.targetTouches[0]&&t.targetTouches[0].clientY||0}W.DEFAULTS={width:"400px",height:"400px",filterLabel:"time",legendLabel:"value",xLabel:"x",yLabel:"y",zLabel:"z",xValueLabel:function(t){return t},yValueLabel:function(t){return t},zValueLabel:function(t){return t},showXAxis:!0,showYAxis:!0,showZAxis:!0,showGrayBottom:!1,showGrid:!0,showPerspective:!0,showShadow:!1,showSurfaceGrid:!0,keepAspectRatio:!0,rotateAxisLabels:!0,verticalRatio:.5,dotSizeRatio:.02,dotSizeMinFraction:.5,dotSizeMaxFraction:2.5,showAnimationControls:O,animationInterval:1e3,animationPreload:!1,animationAutoStart:O,axisFontSize:14,axisFontType:"arial",axisColor:"#4D4D4D",gridColor:"#D3D3D3",xCenter:"55%",yCenter:"50%",style:W.STYLE.DOT,tooltip:!1,tooltipDelay:300,tooltipStyle:{content:{padding:"10px",border:"1px solid #4d4d4d",color:"#1a1a1a",background:"rgba(255,255,255,0.7)",borderRadius:"2px",boxShadow:"5px 5px 10px rgba(128,128,128,0.5)"},line:{height:"40px",width:"0",borderLeft:"1px solid #4d4d4d",pointerEvents:"none"},dot:{height:"0",width:"0",border:"5px solid #4d4d4d",borderRadius:"5px",pointerEvents:"none"}},dataColor:{fill:"#7DC1FF",stroke:"#3267D2",strokeWidth:1},surfaceColors:O,colormap:O,cameraPosition:{horizontal:1,vertical:.5,distance:1.7},zoomable:!0,ctrlToZoom:!1,showLegend:O,backgroundColor:O,xBarWidth:O,yBarWidth:O,valueMin:O,valueMax:O,xMin:O,xMax:O,xStep:O,yMin:O,yMax:O,yStep:O,zMin:O,zMax:O,zStep:O},t(W.prototype),W.prototype._setScale=function(){this.scale=new r(1/this.xRange.range(),1/this.yRange.range(),1/this.zRange.range()),this.keepAspectRatio&&(this.scale.x<this.scale.y?this.scale.y=this.scale.x:this.scale.x=this.scale.y),this.scale.z*=this.verticalRatio,void 0!==this.valueRange&&(this.scale.value=1/this.valueRange.range());const t=this.xRange.center()*this.scale.x,e=this.yRange.center()*this.scale.y,i=this.zRange.center()*this.scale.z;this.camera.setArmLocation(t,e,i)},W.prototype._convert3Dto2D=function(t){const e=this._convertPointToTranslation(t);return this._convertTranslationToScreen(e)},W.prototype._convertPointToTranslation=function(t){const e=this.camera.getCameraLocation(),i=this.camera.getCameraRotation(),o=t.x*this.scale.x,n=t.y*this.scale.y,s=t.z*this.scale.z,a=e.x,h=e.y,l=e.z,c=Math.sin(i.x),d=Math.cos(i.x),u=Math.sin(i.y),p=Math.cos(i.y),f=Math.sin(i.z),m=Math.cos(i.z);return new r(p*(f*(n-h)+m*(o-a))-u*(s-l),c*(p*(s-l)+u*(f*(n-h)+m*(o-a)))+d*(m*(n-h)-f*(o-a)),d*(p*(s-l)+u*(f*(n-h)+m*(o-a)))-c*(m*(n-h)-f*(o-a)))},W.prototype._convertTranslationToScreen=function(t){const e=this.eye.x,i=this.eye.y,o=this.eye.z,n=t.x,s=t.y,a=t.z;let r,l;return this.showPerspective?(r=o/a*(n-e),l=o/a*(s-i)):(r=n*(-o/this.camera.getArmLength()),l=s*(-o/this.camera.getArmLength())),new h(this.currentXCenter+r*this.frame.canvas.clientWidth,this.currentYCenter-l*this.frame.canvas.clientWidth)},W.prototype._calcTranslations=function(t){for(let e=0;e<t.length;e++){const i=t[e];i.trans=this._convertPointToTranslation(i.point),i.screen=this._convertTranslationToScreen(i.trans);const o=this._convertPointToTranslation(i.bottom);i.dist=this.showPerspective?o.length():-o.z}t.sort((function(t,e){return e.dist-t.dist}))},W.prototype._initializeRanges=function(){const t=this.dataGroup;this.xRange=t.xRange,this.yRange=t.yRange,this.zRange=t.zRange,this.valueRange=t.valueRange,this.xStep=t.xStep,this.yStep=t.yStep,this.zStep=t.zStep,this.xBarWidth=t.xBarWidth,this.yBarWidth=t.yBarWidth,this.colX=t.colX,this.colY=t.colY,this.colZ=t.colZ,this.colValue=t.colValue,this._setScale()},W.prototype.getDataPoints=function(t){const e=[];for(let i=0;i<t.length;i++){const o=new r;o.x=t[i][this.colX]||0,o.y=t[i][this.colY]||0,o.z=t[i][this.colZ]||0,o.data=t[i],o.value=t[i][this.colValue]||0;const n={};n.point=o,n.bottom=new r(o.x,o.y,this.zRange.min),n.trans=void 0,n.screen=void 0,e.push(n)}return e},W.prototype._getDataPoints=function(t){let e,i,o,n,s=[];if(this.style===W.STYLE.GRID||this.style===W.STYLE.SURFACE){const a=this.dataGroup.getDistinctValues(this.colX,t),r=this.dataGroup.getDistinctValues(this.colY,t);s=this.getDataPoints(t);const h=[];for(o=0;o<s.length;o++){n=s[o];const t=a.indexOf(n.point.x),e=r.indexOf(n.point.y);void 0===h[t]&&(h[t]=[]),h[t][e]=n}for(e=0;e<h.length;e++)for(i=0;i<h[e].length;i++)h[e][i]&&(h[e][i].pointRight=e<h.length-1?h[e+1][i]:void 0,h[e][i].pointTop=i<h[e].length-1?h[e][i+1]:void 0,h[e][i].pointCross=e<h.length-1&&i<h[e].length-1?h[e+1][i+1]:void 0)}else if(s=this.getDataPoints(t),this.style===W.STYLE.LINE)for(o=0;o<s.length;o++)o>0&&(s[o-1].pointNext=s[o]);return s},W.prototype.create=function(){for(;this.containerElement.hasChildNodes();)this.containerElement.removeChild(this.containerElement.firstChild);this.frame=document.createElement("div"),this.frame.style.position="relative",this.frame.style.overflow="hidden",this.frame.canvas=document.createElement("canvas"),this.frame.canvas.style.position="relative",this.frame.appendChild(this.frame.canvas);{const t=document.createElement("DIV");t.style.color="red",t.style.fontWeight="bold",t.style.padding="10px",t.innerHTML="Error: your browser does not support HTML canvas",this.frame.canvas.appendChild(t)}this.frame.filter=document.createElement("div"),this.frame.filter.style.position="absolute",this.frame.filter.style.bottom="0px",this.frame.filter.style.left="0px",this.frame.filter.style.width="100%",this.frame.appendChild(this.frame.filter);const t=this;e.addEventListener(this.frame.canvas,"mousedown",(function(e){t._onMouseDown(e)})),e.addEventListener(this.frame.canvas,"touchstart",(function(e){t._onTouchStart(e)})),e.addEventListener(this.frame.canvas,"mousewheel",(function(e){t._onWheel(e)})),e.addEventListener(this.frame.canvas,"mousemove",(function(e){t._onTooltip(e)})),e.addEventListener(this.frame.canvas,"click",(function(e){t._onClick(e)})),this.containerElement.appendChild(this.frame)},W.prototype._setSize=function(t,e){this.frame.style.width=t,this.frame.style.height=e,this._resizeCanvas()},W.prototype._resizeCanvas=function(){this.frame.canvas.style.width="100%",this.frame.canvas.style.height="100%",this.frame.canvas.width=this.frame.canvas.clientWidth,this.frame.canvas.height=this.frame.canvas.clientHeight,this.frame.filter.style.width=this.frame.canvas.clientWidth-20+"px"},W.prototype.animationStart=function(){if(this.animationAutoStart&&this.dataGroup.dataFilter){if(!this.frame.filter||!this.frame.filter.slider)throw new Error("No animation available");this.frame.filter.slider.play()}},W.prototype.animationStop=function(){this.frame.filter&&this.frame.filter.slider&&this.frame.filter.slider.stop()},W.prototype._resizeCenter=function(){"%"===this.xCenter.charAt(this.xCenter.length-1)?this.currentXCenter=parseFloat(this.xCenter)/100*this.frame.canvas.clientWidth:this.currentXCenter=parseFloat(this.xCenter),"%"===this.yCenter.charAt(this.yCenter.length-1)?this.currentYCenter=parseFloat(this.yCenter)/100*(this.frame.canvas.clientHeight-this.frame.filter.clientHeight):this.currentYCenter=parseFloat(this.yCenter)},W.prototype.getCameraPosition=function(){const t=this.camera.getArmRotation();return t.distance=this.camera.getArmLength(),t},W.prototype._readData=function(t){this.dataPoints=this.dataGroup.initializeData(this,t,this.style),this._initializeRanges(),this._redrawFilter()},W.prototype.setData=function(t){null!=t&&(this._readData(t),this.redraw(),this.animationStart())},W.prototype.setOptions=function(t){if(void 0===t)return;!0===i.validate(t,P)&&console.error("%cErrors have been found in the supplied options object.",o),this.animationStop(),function(t,e){if(void 0!==t){if(void 0===e)throw new Error("No dst passed");if(void 0===g||x(g))throw new Error("DEFAULTS not set for module Settings");b(t,e,m),b(t,e,y,"default"),_(t,e)}}(t,this),this.setPointDrawingMethod(),this._setSize(this.width,this.height),this.setAxisLabelMethod(),this.setData(this.dataGroup.getDataTable()),this.animationStart()},W.prototype.setPointDrawingMethod=function(){let t;switch(this.style){case W.STYLE.BAR:t=this._redrawBarGraphPoint;break;case W.STYLE.BARCOLOR:t=this._redrawBarColorGraphPoint;break;case W.STYLE.BARSIZE:t=this._redrawBarSizeGraphPoint;break;case W.STYLE.DOT:t=this._redrawDotGraphPoint;break;case W.STYLE.DOTLINE:t=this._redrawDotLineGraphPoint;break;case W.STYLE.DOTCOLOR:t=this._redrawDotColorGraphPoint;break;case W.STYLE.DOTSIZE:t=this._redrawDotSizeGraphPoint;break;case W.STYLE.SURFACE:t=this._redrawSurfaceGraphPoint;break;case W.STYLE.GRID:t=this._redrawGridGraphPoint;break;case W.STYLE.LINE:t=this._redrawLineGraphPoint;break;default:throw new Error("Can not determine point drawing method for graph style '"+this.style+"'")}this._pointDrawingMethod=t},W.prototype.setAxisLabelMethod=function(){this.rotateAxisLabels?(this._drawAxisLabelX=this.drawAxisLabelXRotate,this._drawAxisLabelY=this.drawAxisLabelYRotate,this._drawAxisLabelZ=this.drawAxisLabelZRotate):(this._drawAxisLabelX=this.drawAxisLabelX,this._drawAxisLabelY=this.drawAxisLabelY,this._drawAxisLabelZ=this.drawAxisLabelZ)},W.prototype.redraw=function(){if(void 0===this.dataPoints)throw new Error("Graph data not initialized");this._resizeCanvas(),this._resizeCenter(),this._redrawSlider(),this._redrawClear(),this._redrawAxis(),this._redrawDataGraph(),this._redrawInfo(),this._redrawLegend()},W.prototype._getContext=function(){const t=this.frame.canvas.getContext("2d");return t.lineJoin="round",t.lineCap="round",t},W.prototype._redrawClear=function(){const t=this.frame.canvas;t.getContext("2d").clearRect(0,0,t.width,t.height)},W.prototype._dotSize=function(){return this.frame.clientWidth*this.dotSizeRatio},W.prototype._getLegendWidth=function(){let t;if(this.style===W.STYLE.DOTSIZE){t=this._dotSize()*this.dotSizeMaxFraction}else t=this.style===W.STYLE.BARSIZE?this.xBarWidth:20;return t},W.prototype._redrawLegend=function(){if(!0!==this.showLegend)return;if(this.style===W.STYLE.LINE||this.style===W.STYLE.BARSIZE)return;const t=this.style===W.STYLE.BARSIZE||this.style===W.STYLE.DOTSIZE,e=this.style===W.STYLE.DOTSIZE||this.style===W.STYLE.DOTCOLOR||this.style===W.STYLE.SURFACE||this.style===W.STYLE.BARCOLOR,i=Math.max(.25*this.frame.clientHeight,100),o=this.margin,n=this._getLegendWidth(),s=this.frame.clientWidth-this.margin,a=s-n,r=o+i,l=this._getContext();if(l.lineWidth=1,l.font="14px arial",!1===t){const t=0,e=i;let r;for(r=t;r<e;r++){const i=1-(r-t)/(e-t),n=this._colormap(i,1);l.strokeStyle=n,l.beginPath(),l.moveTo(a,o+r),l.lineTo(s,o+r),l.stroke()}l.strokeStyle=this.axisColor,l.strokeRect(a,o,n,i)}else{let t;this.style===W.STYLE.DOTSIZE?t=n*(this.dotSizeMinFraction/this.dotSizeMaxFraction):(this.style,W.STYLE.BARSIZE),l.strokeStyle=this.axisColor,l.fillStyle=this.dataColor.fill,l.beginPath(),l.moveTo(a,o),l.lineTo(s,o),l.lineTo(a+t,r),l.lineTo(a,r),l.closePath(),l.fill(),l.stroke()}const c=e?this.valueRange.min:this.zRange.min,u=e?this.valueRange.max:this.zRange.max,p=new d(c,u,(u-c)/5,!0);for(p.start(!0);!p.end();){const t=r-(p.getCurrent()-c)/(u-c)*i,e=new h(a-5,t),o=new h(a,t);this._line(l,e,o),l.textAlign="right",l.textBaseline="middle",l.fillStyle=this.axisColor,l.fillText(p.getCurrent(),a-10,t),p.next()}l.textAlign="right",l.textBaseline="top";const f=this.legendLabel;l.fillText(f,s,r+this.margin)},W.prototype._redrawFilter=function(){const t=this.dataGroup.dataFilter,e=this.frame.filter;if(e.innerHTML="",!t)return void(e.slider=void 0);const i=new l(e,{visible:this.showAnimationControls});e.slider=i,e.style.padding="10px",i.setValues(t.values),i.setPlayInterval(this.animationInterval);const o=this;i.setOnChangeCallback((function(){const t=o.dataGroup.dataFilter,e=i.getIndex();t.selectValue(e),o.dataPoints=t._getDataPoints(),o.redraw()}))},W.prototype._redrawSlider=function(){void 0!==this.frame.filter.slider&&this.frame.filter.slider.redraw()},W.prototype._redrawInfo=function(){const t=this.dataGroup.getInfo();if(void 0===t)return;const e=this._getContext();e.font="14px arial",e.lineStyle="gray",e.fillStyle="gray",e.textAlign="left",e.textBaseline="top";const i=this.margin,o=this.margin;e.fillText(t,i,o)},W.prototype._line=function(t,e,i,o){void 0!==o&&(t.strokeStyle=o),t.beginPath(),t.moveTo(e.x,e.y),t.lineTo(i.x,i.y),t.stroke()},W.prototype.drawAxisLabelX=function(t,e,i,o,n){void 0===n&&(n=0);const s=this._convert3Dto2D(e);Math.cos(2*o)>0?(t.textAlign="center",t.textBaseline="top",s.y+=n):Math.sin(2*o)<0?(t.textAlign="right",t.textBaseline="middle"):(t.textAlign="left",t.textBaseline="middle"),t.fillStyle=this.axisColor,t.fillText(i,s.x,s.y)},W.prototype.drawAxisLabelY=function(t,e,i,o,n){void 0===n&&(n=0);const s=this._convert3Dto2D(e);Math.cos(2*o)<0?(t.textAlign="center",t.textBaseline="top",s.y+=n):Math.sin(2*o)>0?(t.textAlign="right",t.textBaseline="middle"):(t.textAlign="left",t.textBaseline="middle"),t.fillStyle=this.axisColor,t.fillText(i,s.x,s.y)},W.prototype.drawAxisLabelZ=function(t,e,i,o){void 0===o&&(o=0);const n=this._convert3Dto2D(e);t.textAlign="right",t.textBaseline="middle",t.fillStyle=this.axisColor,t.fillText(i,n.x-o,n.y)},W.prototype.drawAxisLabelXRotate=function(t,e,i,o,n){const s=this._convert3Dto2D(e);Math.cos(2*o)>0?(t.save(),t.translate(s.x,s.y),t.rotate(-Math.PI/2),t.textAlign="right",t.textBaseline="middle",t.fillStyle=this.axisColor,t.fillText(i,0,0),t.restore()):Math.sin(2*o)<0?(t.textAlign="right",t.textBaseline="middle",t.fillStyle=this.axisColor,t.fillText(i,s.x,s.y)):(t.textAlign="left",t.textBaseline="middle",t.fillStyle=this.axisColor,t.fillText(i,s.x,s.y))},W.prototype.drawAxisLabelYRotate=function(t,e,i,o,n){const s=this._convert3Dto2D(e);Math.cos(2*o)<0?(t.save(),t.translate(s.x,s.y),t.rotate(-Math.PI/2),t.textAlign="right",t.textBaseline="middle",t.fillStyle=this.axisColor,t.fillText(i,0,0),t.restore()):Math.sin(2*o)>0?(t.textAlign="right",t.textBaseline="middle",t.fillStyle=this.axisColor,t.fillText(i,s.x,s.y)):(t.textAlign="left",t.textBaseline="middle",t.fillStyle=this.axisColor,t.fillText(i,s.x,s.y))},W.prototype.drawAxisLabelZRotate=function(t,e,i,o){void 0===o&&(o=0);const n=this._convert3Dto2D(e);t.textAlign="right",t.textBaseline="middle",t.fillStyle=this.axisColor,t.fillText(i,n.x-o,n.y)},W.prototype._line3d=function(t,e,i,o){const n=this._convert3Dto2D(e),s=this._convert3Dto2D(i);this._line(t,n,s,o)},W.prototype._redrawAxis=function(){const t=this._getContext();let e,i,o,n,s,a,l,c,u,p,f;t.font=this.axisFontSize/this.camera.getArmLength()+"px "+this.axisFontType;const m=.025/this.scale.x,y=.025/this.scale.y,g=5/this.camera.getArmLength(),x=this.camera.getArmRotation().horizontal,v=new h(Math.cos(x),Math.sin(x)),w=this.xRange,b=this.yRange,_=this.zRange;let C;for(t.lineWidth=1,n=void 0===this.defaultXStep,o=new d(w.min,w.max,this.xStep,n),o.start(!0);!o.end();){const n=o.getCurrent();if(this.showGrid?(e=new r(n,b.min,_.min),i=new r(n,b.max,_.min),this._line3d(t,e,i,this.gridColor)):this.showXAxis&&(e=new r(n,b.min,_.min),i=new r(n,b.min+m,_.min),this._line3d(t,e,i,this.axisColor),e=new r(n,b.max,_.min),i=new r(n,b.max-m,_.min),this._line3d(t,e,i,this.axisColor)),this.showXAxis){l=v.x>0?b.min:b.max,C=new r(n,l,_.min);const e="  "+this.xValueLabel(n)+"  ";this._drawAxisLabelX.call(this,t,C,e,x,g)}o.next()}for(t.lineWidth=1,n=void 0===this.defaultYStep,o=new d(b.min,b.max,this.yStep,n),o.start(!0);!o.end();){const n=o.getCurrent();if(this.showGrid?(e=new r(w.min,n,_.min),i=new r(w.max,n,_.min),this._line3d(t,e,i,this.gridColor)):this.showYAxis&&(e=new r(w.min,n,_.min),i=new r(w.min+y,n,_.min),this._line3d(t,e,i,this.axisColor),e=new r(w.max,n,_.min),i=new r(w.max-y,n,_.min),this._line3d(t,e,i,this.axisColor)),this.showYAxis){a=v.y>0?w.min:w.max,C=new r(a,n,_.min);const e="  "+this.yValueLabel(n)+"  ";this._drawAxisLabelY.call(this,t,C,e,x,g)}o.next()}if(this.showZAxis){for(t.lineWidth=1,n=void 0===this.defaultZStep,o=new d(_.min,_.max,this.zStep,n),o.start(!0),a=v.x>0?w.min:w.max,l=v.y<0?b.min:b.max;!o.end();){const e=o.getCurrent(),n=new r(a,l,e),s=this._convert3Dto2D(n);i=new h(s.x-g,s.y),this._line(t,s,i,this.axisColor);const c=this.zValueLabel(e)+" ";this._drawAxisLabelZ.call(this,t,n,c,5),o.next()}t.lineWidth=1,e=new r(a,l,_.min),i=new r(a,l,_.max),this._line3d(t,e,i,this.axisColor)}if(this.showXAxis){let e,i;t.lineWidth=1,e=new r(w.min,b.min,_.min),i=new r(w.max,b.min,_.min),this._line3d(t,e,i,this.axisColor),e=new r(w.min,b.max,_.min),i=new r(w.max,b.max,_.min),this._line3d(t,e,i,this.axisColor)}this.showYAxis&&(t.lineWidth=1,e=new r(w.min,b.min,_.min),i=new r(w.min,b.max,_.min),this._line3d(t,e,i,this.axisColor),e=new r(w.max,b.min,_.min),i=new r(w.max,b.max,_.min),this._line3d(t,e,i,this.axisColor));const L=this.xLabel;L.length>0&&this.showXAxis&&(f=.1/this.scale.y,a=(w.max+3*w.min)/4,l=v.x>0?b.min-f:b.max+f,s=new r(a,l,_.min),this.drawAxisLabelX(t,s,L,x));const S=this.yLabel;S.length>0&&this.showYAxis&&(p=.1/this.scale.x,a=v.y>0?w.min-p:w.max+p,l=(b.max+3*b.min)/4,s=new r(a,l,_.min),this.drawAxisLabelY(t,s,S,x));const T=this.zLabel;T.length>0&&this.showZAxis&&(u=30,a=v.x>0?w.min:w.max,l=v.y<0?b.min:b.max,c=(_.max+3*_.min)/4,s=new r(a,l,c),this.drawAxisLabelZ(t,s,T,30))},W.prototype._getStrokeWidth=function(t){return void 0!==t?this.showPerspective?1/-t.trans.z*this.dataColor.strokeWidth:-this.eye.z/this.camera.getArmLength()*this.dataColor.strokeWidth:this.dataColor.strokeWidth},W.prototype._redrawBar=function(t,e,i,o,n,s){let a;const h=this,l=e.point,c=this.zRange.min,d=[{point:new r(l.x-i,l.y-o,l.z)},{point:new r(l.x+i,l.y-o,l.z)},{point:new r(l.x+i,l.y+o,l.z)},{point:new r(l.x-i,l.y+o,l.z)}],u=[{point:new r(l.x-i,l.y-o,c)},{point:new r(l.x+i,l.y-o,c)},{point:new r(l.x+i,l.y+o,c)},{point:new r(l.x-i,l.y+o,c)}];d.forEach((function(t){t.screen=h._convert3Dto2D(t.point)})),u.forEach((function(t){t.screen=h._convert3Dto2D(t.point)}));const p=[{corners:d,center:r.avg(u[0].point,u[2].point)},{corners:[d[0],d[1],u[1],u[0]],center:r.avg(u[1].point,u[0].point)},{corners:[d[1],d[2],u[2],u[1]],center:r.avg(u[2].point,u[1].point)},{corners:[d[2],d[3],u[3],u[2]],center:r.avg(u[3].point,u[2].point)},{corners:[d[3],d[0],u[0],u[3]],center:r.avg(u[0].point,u[3].point)}];e.surfaces=p;for(let t=0;t<p.length;t++){a=p[t];const e=this._convertPointToTranslation(a.center);a.dist=this.showPerspective?e.length():-e.z}p.sort((function(t,e){const i=e.dist-t.dist;return i||(t.corners===d?1:e.corners===d?-1:0)})),t.lineWidth=this._getStrokeWidth(e),t.strokeStyle=s,t.fillStyle=n;for(let e=2;e<p.length;e++)a=p[e],this._polygon(t,a.corners)},W.prototype._polygon=function(t,e,i,o){if(!(e.length<2)){void 0!==i&&(t.fillStyle=i),void 0!==o&&(t.strokeStyle=o),t.beginPath(),t.moveTo(e[0].screen.x,e[0].screen.y);for(let i=1;i<e.length;++i){const o=e[i];t.lineTo(o.screen.x,o.screen.y)}t.closePath(),t.fill(),t.stroke()}},W.prototype._drawCircle=function(t,e,i,o,n){const s=this._calcRadius(e,n);t.lineWidth=this._getStrokeWidth(e),t.strokeStyle=o,t.fillStyle=i,t.beginPath(),t.arc(e.screen.x,e.screen.y,s,0,2*Math.PI,!0),t.fill(),t.stroke()},W.prototype._getColorsRegular=function(t){const e=(t.point.value-this.valueRange.min)*this.scale.value;return{fill:this._colormap(e,1),border:this._colormap(e,.8)}},W.prototype._getColorsColor=function(t){let e,i,o;if(t&&t.point&&t.point.data&&t.point.data.style&&(o=t.point.data.style),o&&"object"==typeof o&&o.fill&&o.stroke)return{fill:o.fill,border:o.stroke};if("string"==typeof t.point.value)e=t.point.value,i=t.point.value;else{const o=(t.point.value-this.valueRange.min)*this.scale.value;e=this._colormap(o,1),i=this._colormap(o,.8)}return{fill:e,border:i}},W.prototype._getColorsSize=function(){return{fill:this.dataColor.fill,border:this.dataColor.stroke}},W.prototype._colormap=function(t,i=1){let o,n,s,a;const r=this.colormap;if(Array.isArray(r)){const e=r.length-1,i=Math.max(Math.floor(t*e),0),a=Math.min(i+1,e),h=t*e-i,l=r[i],c=r[a];o=l.r+h*(c.r-l.r),n=l.g+h*(c.g-l.g),s=l.b+h*(c.b-l.b)}else if("function"==typeof r)({r:o,g:n,b:s,a:a}=r(t));else{const i=240*(1-t);({r:o,g:n,b:s}=e.HSVToRGB(i/360,1,1))}return"number"!=typeof a||Number.isNaN(a)?`RGB(${Math.round(o*i)}, ${Math.round(n*i)}, ${Math.round(s*i)})`:`RGBA(${Math.round(o*i)}, ${Math.round(n*i)}, ${Math.round(s*i)}, ${a})`},W.prototype._calcRadius=function(t,e){let i;return void 0===e&&(e=this._dotSize()),i=this.showPerspective?e/-t.trans.z:e*(-this.eye.z/this.camera.getArmLength()),i<0&&(i=0),i},W.prototype._redrawBarGraphPoint=function(t,e){const i=this.xBarWidth/2,o=this.yBarWidth/2,n=this._getColorsRegular(e);this._redrawBar(t,e,i,o,n.fill,n.border)},W.prototype._redrawBarColorGraphPoint=function(t,e){const i=this.xBarWidth/2,o=this.yBarWidth/2,n=this._getColorsColor(e);this._redrawBar(t,e,i,o,n.fill,n.border)},W.prototype._redrawBarSizeGraphPoint=function(t,e){const i=(e.point.value-this.valueRange.min)/this.valueRange.range(),o=this.xBarWidth/2*(.8*i+.2),n=this.yBarWidth/2*(.8*i+.2),s=this._getColorsSize();this._redrawBar(t,e,o,n,s.fill,s.border)},W.prototype._redrawDotGraphPoint=function(t,e){const i=this._getColorsRegular(e);this._drawCircle(t,e,i.fill,i.border)},W.prototype._redrawDotLineGraphPoint=function(t,e){const i=this._convert3Dto2D(e.bottom);t.lineWidth=1,this._line(t,i,e.screen,this.gridColor),this._redrawDotGraphPoint(t,e)},W.prototype._redrawDotColorGraphPoint=function(t,e){const i=this._getColorsColor(e);this._drawCircle(t,e,i.fill,i.border)},W.prototype._redrawDotSizeGraphPoint=function(t,e){const i=this._dotSize(),o=(e.point.value-this.valueRange.min)/this.valueRange.range(),n=i*this.dotSizeMinFraction,s=n+(i*this.dotSizeMaxFraction-n)*o,a=this._getColorsSize();this._drawCircle(t,e,a.fill,a.border,s)},W.prototype._redrawSurfaceGraphPoint=function(t,e){const i=e.pointRight,o=e.pointTop,n=e.pointCross;if(void 0===e||void 0===i||void 0===o||void 0===n)return;let s,a,h,l=!0;if(this.showGrayBottom||this.showShadow){const t=r.subtract(n.trans,e.trans),s=r.subtract(o.trans,i.trans),a=r.crossProduct(t,s);if(this.showPerspective){const t=r.avg(r.avg(e.trans,n.trans),r.avg(i.trans,o.trans));h=-r.dotProduct(a.normalize(),t.normalize())}else h=a.z/a.length();l=h>0}if(l||!this.showGrayBottom){const t=((e.point.value+i.point.value+o.point.value+n.point.value)/4-this.valueRange.min)*this.scale.value,a=this.showShadow?(1+h)/2:1;s=this._colormap(t,a)}else s="gray";a=this.showSurfaceGrid?this.axisColor:s,t.lineWidth=this._getStrokeWidth(e);const c=[e,i,n,o];this._polygon(t,c,s,a)},W.prototype._drawGridLine=function(t,e,i){if(void 0===e||void 0===i)return;const o=((e.point.value+i.point.value)/2-this.valueRange.min)*this.scale.value;t.lineWidth=2*this._getStrokeWidth(e),t.strokeStyle=this._colormap(o,1),this._line(t,e.screen,i.screen)},W.prototype._redrawGridGraphPoint=function(t,e){this._drawGridLine(t,e,e.pointRight),this._drawGridLine(t,e,e.pointTop)},W.prototype._redrawLineGraphPoint=function(t,e){void 0!==e.pointNext&&(t.lineWidth=this._getStrokeWidth(e),t.strokeStyle=this.dataColor.stroke,this._line(t,e.screen,e.pointNext.screen))},W.prototype._redrawDataGraph=function(){const t=this._getContext();let e;if(!(void 0===this.dataPoints||this.dataPoints.length<=0))for(this._calcTranslations(this.dataPoints),e=0;e<this.dataPoints.length;e++){const i=this.dataPoints[e];this._pointDrawingMethod.call(this,t,i)}},W.prototype._storeMousePosition=function(t){this.startMouseX=G(t),this.startMouseY=Y(t),this._startCameraOffset=this.camera.getOffset()},W.prototype._onMouseDown=function(t){if(t=t||window.event,this.leftButtonDown&&this._onMouseUp(t),this.leftButtonDown=t.which?1===t.which:1===t.button,!this.leftButtonDown&&!this.touchDown)return;this._storeMousePosition(t),this.startStart=new Date(this.start),this.startEnd=new Date(this.end),this.startArmRotation=this.camera.getArmRotation(),this.frame.style.cursor="move";const i=this;this.onmousemove=function(t){i._onMouseMove(t)},this.onmouseup=function(t){i._onMouseUp(t)},e.addEventListener(document,"mousemove",i.onmousemove),e.addEventListener(document,"mouseup",i.onmouseup),e.preventDefault(t)},W.prototype._onMouseMove=function(t){this.moving=!0,t=t||window.event;const i=parseFloat(G(t))-this.startMouseX,o=parseFloat(Y(t))-this.startMouseY;if(t&&!0===t.ctrlKey){const e=.5*this.frame.clientWidth,n=.5*this.frame.clientHeight,s=(this._startCameraOffset.x||0)-i/e*this.camera.armLength*.8,a=(this._startCameraOffset.y||0)+o/n*this.camera.armLength*.8;this.camera.setOffset(s,a),this._storeMousePosition(t)}else{let t=this.startArmRotation.horizontal+i/200,e=this.startArmRotation.vertical+o/200;const n=4,s=Math.sin(n/360*2*Math.PI);Math.abs(Math.sin(t))<s&&(t=Math.round(t/Math.PI)*Math.PI-.001),Math.abs(Math.cos(t))<s&&(t=(Math.round(t/Math.PI-.5)+.5)*Math.PI-.001),Math.abs(Math.sin(e))<s&&(e=Math.round(e/Math.PI)*Math.PI),Math.abs(Math.cos(e))<s&&(e=(Math.round(e/Math.PI-.5)+.5)*Math.PI),this.camera.setArmRotation(t,e)}this.redraw();const n=this.getCameraPosition();this.emit("cameraPositionChange",n),e.preventDefault(t)},W.prototype._onMouseUp=function(t){this.frame.style.cursor="auto",this.leftButtonDown=!1,e.removeEventListener(document,"mousemove",this.onmousemove),e.removeEventListener(document,"mouseup",this.onmouseup),e.preventDefault(t)},W.prototype._onClick=function(t){if(this.onclick_callback||this.hasListeners("click")){if(this.moving)this.moving=!1;else{const e=this.frame.getBoundingClientRect(),i=G(t)-e.left,o=Y(t)-e.top,n=this._dataPointFromXY(i,o);n&&(this.onclick_callback&&this.onclick_callback(n.point.data),this.emit("click",n.point.data))}e.preventDefault(t)}},W.prototype._onTooltip=function(t){const e=this.tooltipDelay,i=this.frame.getBoundingClientRect(),o=G(t)-i.left,n=Y(t)-i.top;if(this.showTooltip)if(this.tooltipTimeout&&clearTimeout(this.tooltipTimeout),this.leftButtonDown)this._hideTooltip();else if(this.tooltip&&this.tooltip.dataPoint){const t=this._dataPointFromXY(o,n);t!==this.tooltip.dataPoint&&(t?this._showTooltip(t):this._hideTooltip())}else{const t=this;this.tooltipTimeout=setTimeout((function(){t.tooltipTimeout=null;const e=t._dataPointFromXY(o,n);e&&t._showTooltip(e)}),e)}},W.prototype._onTouchStart=function(t){this.touchDown=!0;const i=this;this.ontouchmove=function(t){i._onTouchMove(t)},this.ontouchend=function(t){i._onTouchEnd(t)},e.addEventListener(document,"touchmove",i.ontouchmove),e.addEventListener(document,"touchend",i.ontouchend),this._onMouseDown(t)},W.prototype._onTouchMove=function(t){this._onMouseMove(t)},W.prototype._onTouchEnd=function(t){this.touchDown=!1,e.removeEventListener(document,"touchmove",this.ontouchmove),e.removeEventListener(document,"touchend",this.ontouchend),this._onMouseUp(t)},W.prototype._onWheel=function(t){if(t||(t=window.event),this.zoomable&&(!this.ctrlToZoom||t.ctrlKey)){let i=0;if(t.wheelDelta?i=t.wheelDelta/120:t.detail&&(i=-t.detail/3),i){const t=this.camera.getArmLength()*(1-i/10);this.camera.setArmLength(t),this.redraw(),this._hideTooltip()}const o=this.getCameraPosition();this.emit("cameraPositionChange",o),e.preventDefault(t)}},W.prototype._insideTriangle=function(t,e){const i=e[0],o=e[1],n=e[2];function s(t){return t>0?1:t<0?-1:0}const a=s((o.x-i.x)*(t.y-i.y)-(o.y-i.y)*(t.x-i.x)),r=s((n.x-o.x)*(t.y-o.y)-(n.y-o.y)*(t.x-o.x)),h=s((i.x-n.x)*(t.y-n.y)-(i.y-n.y)*(t.x-n.x));return!(0!=a&&0!=r&&a!=r||0!=r&&0!=h&&r!=h||0!=a&&0!=h&&a!=h)},W.prototype._dataPointFromXY=function(t,e){const i=new h(t,e);let o,n=null,s=null,a=null;if(this.style===W.STYLE.BAR||this.style===W.STYLE.BARCOLOR||this.style===W.STYLE.BARSIZE)for(o=this.dataPoints.length-1;o>=0;o--){n=this.dataPoints[o];const t=n.surfaces;if(t)for(let e=t.length-1;e>=0;e--){const o=t[e].corners,s=[o[0].screen,o[1].screen,o[2].screen],a=[o[2].screen,o[3].screen,o[0].screen];if(this._insideTriangle(i,s)||this._insideTriangle(i,a))return n}}else for(o=0;o<this.dataPoints.length;o++){n=this.dataPoints[o];const i=n.screen;if(i){const o=Math.abs(t-i.x),r=Math.abs(e-i.y),h=Math.sqrt(o*o+r*r);(null===a||h<a)&&h<100&&(a=h,s=n)}}return s},W.prototype.hasBars=function(t){return t==W.STYLE.BAR||t==W.STYLE.BARCOLOR||t==W.STYLE.BARSIZE},W.prototype._showTooltip=function(t){let e,i,o;this.tooltip?(e=this.tooltip.dom.content,i=this.tooltip.dom.line,o=this.tooltip.dom.dot):(e=document.createElement("div"),Object.assign(e.style,{},this.tooltipStyle.content),e.style.position="absolute",i=document.createElement("div"),Object.assign(i.style,{},this.tooltipStyle.line),i.style.position="absolute",o=document.createElement("div"),Object.assign(o.style,{},this.tooltipStyle.dot),o.style.position="absolute",this.tooltip={dataPoint:null,dom:{content:e,line:i,dot:o}}),this._hideTooltip(),this.tooltip.dataPoint=t,"function"==typeof this.showTooltip?e.innerHTML=this.showTooltip(t.point):e.innerHTML="<table><tr><td>"+this.xLabel+":</td><td>"+t.point.x+"</td></tr><tr><td>"+this.yLabel+":</td><td>"+t.point.y+"</td></tr><tr><td>"+this.zLabel+":</td><td>"+t.point.z+"</td></tr></table>",e.style.left="0",e.style.top="0",this.frame.appendChild(e),this.frame.appendChild(i),this.frame.appendChild(o);const n=e.offsetWidth,s=e.offsetHeight,a=i.offsetHeight,r=o.offsetWidth,h=o.offsetHeight;let l=t.screen.x-n/2;l=Math.min(Math.max(l,10),this.frame.clientWidth-10-n),i.style.left=t.screen.x+"px",i.style.top=t.screen.y-a+"px",e.style.left=l+"px",e.style.top=t.screen.y-a-s+"px",o.style.left=t.screen.x-r/2+"px",o.style.top=t.screen.y-h/2+"px"},W.prototype._hideTooltip=function(){if(this.tooltip){this.tooltip.dataPoint=null;for(const t in this.tooltip.dom)if(Object.prototype.hasOwnProperty.call(this.tooltip.dom,t)){const e=this.tooltip.dom[t];e&&e.parentNode&&e.parentNode.removeChild(e)}}},W.prototype.setCameraPosition=function(t){S(t,this),this.redraw()},W.prototype.setSize=function(t,e){this._setSize(t,e),this.redraw()};export{W as Graph3d,u as Graph3dCamera,B as Graph3dFilter,h as Graph3dPoint2d,r as Graph3dPoint3d,l as Graph3dSlider,d as Graph3dStepNumber};
//# sourceMappingURL=vis-graph3d.min.js.map
